@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -38,6 +38,10 @@
 #include <ext/signature.h>
 #include <ext/safe_renegotiation.h>
 #include <ext/ecc.h>
+#include <ext/status_request.h>
+#include <ext/srtp.h>
+#include <ext/alpn.h>
+#include <ext/new_record_padding.h>
 #include <gnutls_num.h>
 
 
@@ -171,11 +175,6 @@ _gnutls_parse_extensions (gnutls_session_t session,
       type = _gnutls_read_uint16 (&data[pos]);
       pos += 2;
 
-#if 0
-      _gnutls_handshake_log ("EXT[%p]: Found extension '%s/%d'\n", session,
-                         _gnutls_extension_get_name (type), type);
-#endif
-
       if ((ret = _gnutls_extension_list_check (session, type)) < 0)
         {
           gnutls_assert ();
@@ -192,7 +191,12 @@ _gnutls_parse_extensions (gnutls_session_t session,
 
       ext_recv = _gnutls_ext_func_recv (type, parse_type);
       if (ext_recv == NULL)
-        continue;
+        {
+          _gnutls_handshake_log ("EXT[%p]: Found extension '%s/%d'\n", session,
+                                 _gnutls_extension_get_name (type), type);
+
+          continue;
+        }
 
       _gnutls_handshake_log ("EXT[%p]: Parsing extension '%s/%d' (%d bytes)\n",
                          session, _gnutls_extension_get_name (type), type,
@@ -311,10 +315,13 @@ _gnutls_ext_init (void)
   if (ret != GNUTLS_E_SUCCESS)
     return ret;
 
-  ret = _gnutls_ext_register (&ext_mod_cert_type);
+  ret = _gnutls_ext_register (&ext_mod_status_request);
   if (ret != GNUTLS_E_SUCCESS)
     return ret;
 
+  ret = _gnutls_ext_register (&ext_mod_cert_type);
+  if (ret != GNUTLS_E_SUCCESS)
+    return ret;
 
   ret = _gnutls_ext_register (&ext_mod_server_name);
   if (ret != GNUTLS_E_SUCCESS)
@@ -330,9 +337,15 @@ _gnutls_ext_init (void)
     return ret;
 #endif
 
+#ifdef ENABLE_HEARTBEAT
   ret = _gnutls_ext_register (&ext_mod_heartbeat);
   if (ret != GNUTLS_E_SUCCESS)
     return ret;
+#endif
+
+  ret = _gnutls_ext_register (&ext_mod_new_record_padding);
+  if (ret != GNUTLS_E_SUCCESS)
+    return ret;
 
   ret = _gnutls_ext_register (&ext_mod_session_ticket);
   if (ret != GNUTLS_E_SUCCESS)
@@ -350,6 +363,18 @@ _gnutls_ext_init (void)
   if (ret != GNUTLS_E_SUCCESS)
     return ret;
 
+#ifdef ENABLE_DTLS_SRTP
+  ret = _gnutls_ext_register (&ext_mod_srtp);
+  if (ret != GNUTLS_E_SUCCESS)
+    return ret;
+#endif
+
+#ifdef ENABLE_ALPN
+  ret = _gnutls_ext_register (&ext_mod_alpn);
+  if (ret != GNUTLS_E_SUCCESS)
+    return ret;
+#endif
+
   return GNUTLS_E_SUCCESS;
 }
 
@@ -366,7 +391,7 @@ _gnutls_ext_register (extension_entry_st * mod)
 {
   extension_entry_st *p;
 
-  p = gnutls_realloc (extfunc, sizeof (*extfunc) * (extfunc_size + 1));
+  p = gnutls_realloc_fast (extfunc, sizeof (*extfunc) * (extfunc_size + 1));
   if (!p)
     {
       gnutls_assert ();
@@ -382,6 +407,26 @@ _gnutls_ext_register (extension_entry_st * mod)
   return GNUTLS_E_SUCCESS;
 }
 
+int
+_gnutls_ext_before_epoch_change (gnutls_session_t session)
+{
+  unsigned int i;
+  int ret;
+
+  for (i = 0; i < extfunc_size; i++)
+    {
+      if (extfunc[i].epoch_func != NULL)
+        {
+          ret = extfunc[i].epoch_func (session);
+          if (ret < 0)
+            return gnutls_assert_val(ret);
+        }
+    }
+
+  return 0;
+}
+
+
 int
 _gnutls_ext_pack (gnutls_session_t session, gnutls_buffer_st * packed)
 {
@@ -425,7 +470,6 @@ _gnutls_ext_pack (gnutls_session_t session, gnutls_buffer_st * packed)
   _gnutls_write_uint32 (exts, packed->data + total_exts_pos);
 
   return 0;
-
 }
 
 void
