@@ -282,11 +282,13 @@ static const int protocol_priority[] = {
   GNUTLS_TLS1_1,
   GNUTLS_TLS1_0,
   GNUTLS_SSL3,
+  GNUTLS_DTLS1_2,
   GNUTLS_DTLS1_0,
   0
 };
 
 static const int dtls_protocol_priority[] = {
+  GNUTLS_DTLS1_2,
   GNUTLS_DTLS1_0,
   0
 };
@@ -309,18 +311,6 @@ static const int kx_priority_performance[] = {
   0
 };
 
-static const int kx_priority_pfs[] = {
-#ifdef ENABLE_ECDHE
-  GNUTLS_KX_ECDHE_ECDSA,
-  GNUTLS_KX_ECDHE_RSA,
-#endif
-#ifdef ENABLE_DHE
-  GNUTLS_KX_DHE_RSA,
-  GNUTLS_KX_DHE_DSS,
-#endif
-  0
-};
-
 static const int kx_priority_suiteb[] = {
   GNUTLS_KX_ECDHE_ECDSA,
   0
@@ -335,9 +325,6 @@ static const int kx_priority_export[] = {
 #ifdef ENABLE_DHE
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
-#endif
-#ifdef ENABLE_RSA_EXPORT
-  GNUTLS_KX_RSA_EXPORT,
 #endif
   0
 };
@@ -360,27 +347,53 @@ static const int kx_priority_secure[] = {
   GNUTLS_KX_DHE_DSS,
 #endif
   /* GNUTLS_KX_ANON_DH: Man-in-the-middle prone, don't add!
-   * GNUTLS_KX_RSA_EXPORT: Deprecated, don't add!
    */
   0
 };
 
-static const int cipher_priority_performance[] = {
+static const int cipher_priority_performance_sw[] = {
   GNUTLS_CIPHER_ARCFOUR_128,
-  GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
   GNUTLS_CIPHER_3DES_CBC,
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_256_GCM,
+  0
+};
+
+/* If GCM and AES acceleration is available then prefer
+ * them over anything else.
+ */
+static const int cipher_priority_performance_hw_aes[] = {
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_128_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
+  GNUTLS_CIPHER_AES_256_CBC,
+  GNUTLS_CIPHER_ARCFOUR_128,
+  GNUTLS_CIPHER_CAMELLIA_128_CBC,
+  GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_3DES_CBC,
   0
 };
 
-static const int cipher_priority_normal[] = {
-  GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_256_GCM,
+static const int cipher_priority_normal_sw[] = {
   GNUTLS_CIPHER_AES_128_CBC,
+  GNUTLS_CIPHER_CAMELLIA_128_CBC,
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_256_CBC,
+  GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
+  GNUTLS_CIPHER_3DES_CBC,
+  GNUTLS_CIPHER_ARCFOUR_128,
+  0
+};
+
+static const int cipher_priority_normal_hw_aes[] = {
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_128_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
@@ -389,6 +402,10 @@ static const int cipher_priority_normal[] = {
   0
 };
 
+static const int *cipher_priority_performance = cipher_priority_performance_sw;
+static const int *cipher_priority_normal = cipher_priority_normal_sw;
+
+
 static const int cipher_priority_suiteb128[] = {
   GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_AES_256_GCM,
@@ -402,32 +419,30 @@ static const int cipher_priority_suiteb192[] = {
 
 
 static const int cipher_priority_secure128[] = {
-  GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_256_GCM,
-
   GNUTLS_CIPHER_AES_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
+  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
   0
 };
 
 
 static const int cipher_priority_secure192[] = {
-  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
   0
 };
 
 /* The same as cipher_priority_security_normal + arcfour-40. */
 static const int cipher_priority_export[] = {
-  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_AES_128_CBC,
-  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_3DES_CBC,
   GNUTLS_CIPHER_ARCFOUR_128,
   GNUTLS_CIPHER_ARCFOUR_40,
@@ -624,7 +639,6 @@ gnutls_priority_set (gnutls_session_t session, gnutls_priority_t priority)
 
 #define LEVEL_NONE "NONE"
 #define LEVEL_NORMAL "NORMAL"
-#define LEVEL_PFS "PFS"
 #define LEVEL_PERFORMANCE "PERFORMANCE"
 #define LEVEL_SECURE128 "SECURE128"
 #define LEVEL_SECURE192 "SECURE192"
@@ -664,19 +678,6 @@ bulk_rmadd_func *func;
                      sign_priority_default);
       func (&priority_cache->supported_ecc, supported_ecc_normal);
 
-      if (priority_cache->level == 0)
-        priority_cache->level = GNUTLS_SEC_PARAM_VERY_WEAK;
-      return 1;
-    }
-  else if (strcasecmp (level, LEVEL_PFS) == 0)
-    {
-      func (&priority_cache->cipher, cipher_priority_normal);
-      func (&priority_cache->kx, kx_priority_pfs);
-      func (&priority_cache->mac, mac_priority_normal);
-      func (&priority_cache->sign_algo,
-                     sign_priority_default);
-      func (&priority_cache->supported_ecc, supported_ecc_normal);
-
       if (priority_cache->level == 0)
         priority_cache->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       return 1;
@@ -781,10 +782,6 @@ bulk_rmadd_func *func;
  * included as a fallback only.  The ciphers are sorted by security
  * margin.
  *
- * "PFS" means all "secure" ciphersuites that support perfect forward secrecy. 
- * The 256-bit ciphers are included as a fallback only.  
- * The ciphers are sorted by security margin.
- *
  * "SECURE128" means all "secure" ciphersuites of security level 128-bit
  * or more.
  *
@@ -816,7 +813,7 @@ bulk_rmadd_func *func;
  *
  * "NORMAL:-ARCFOUR-128" means normal ciphers except for ARCFOUR-128.
  *
- * "SECURE:-VERS-SSL3.0:+COMP-DEFLATE" means that only secure ciphers are
+ * "SECURE128:-VERS-SSL3.0:+COMP-DEFLATE" means that only secure ciphers are
  * enabled, SSL3.0 is disabled, and libz compression enabled.
  *
  * "NONE:+VERS-TLS-ALL:+AES-128-CBC:+RSA:+SHA1:+COMP-NULL:+SIGN-RSA-SHA1", 
@@ -881,6 +878,7 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
       _set_priority (&(*priority_cache)->cert_type, cert_type_priority_default);
       _set_priority (&(*priority_cache)->sign_algo, sign_priority_default);
       _set_priority (&(*priority_cache)->supported_ecc, supported_ecc_normal);
+      (*priority_cache)->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       i = 0;
     }
   else
@@ -900,7 +898,7 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
           if (broken_list[i][0] == '+')
             {
               fn = prio_add;
-              bulk_fn = _set_priority;
+              bulk_fn = _add_priority;
             }
           else
             {
@@ -1259,6 +1257,15 @@ gnutls_set_default_export_priority (gnutls_session_t session)
   return gnutls_priority_set_direct (session, "EXPORT", NULL);
 }
 
+/* Increases the priority of AES-GCM as it is much faster
+ * than anything else if hardware support is there.
+ */
+void _gnutls_priority_prefer_aes_gcm(void)
+{
+  cipher_priority_performance = cipher_priority_performance_hw_aes;
+  cipher_priority_normal = cipher_priority_normal_hw_aes;
+}
+
 /**
  * gnutls_priority_ecc_curve_list:
  * @pcache: is a #gnutls_prioritity_t structure.
@@ -1280,6 +1287,69 @@ gnutls_priority_ecc_curve_list (gnutls_priority_t pcache, const unsigned int** l
   return pcache->supported_ecc.algorithms;
 }
 
+/**
+ * gnutls_priority_kx_list:
+ * @pcache: is a #gnutls_prioritity_t structure.
+ * @list: will point to an integer list
+ *
+ * Get a list of available key exchange methods in the priority
+ * structure. 
+ *
+ * Returns: the number of curves, or an error code.
+ * Since: 3.2.3
+ **/
+int
+gnutls_priority_kx_list (gnutls_priority_t pcache, const unsigned int** list)
+{
+  if (pcache->kx.algorithms == 0)
+    return 0;
+  
+  *list = pcache->kx.priority;
+  return pcache->kx.algorithms;
+}
+
+/**
+ * gnutls_priority_cipher_list:
+ * @pcache: is a #gnutls_prioritity_t structure.
+ * @list: will point to an integer list
+ *
+ * Get a list of available ciphers in the priority
+ * structure. 
+ *
+ * Returns: the number of curves, or an error code.
+ * Since: 3.2.3
+ **/
+int
+gnutls_priority_cipher_list (gnutls_priority_t pcache, const unsigned int** list)
+{
+  if (pcache->cipher.algorithms == 0)
+    return 0;
+  
+  *list = pcache->cipher.priority;
+  return pcache->cipher.algorithms;
+}
+
+/**
+ * gnutls_priority_mac_list:
+ * @pcache: is a #gnutls_prioritity_t structure.
+ * @list: will point to an integer list
+ *
+ * Get a list of available MAC algorithms in the priority
+ * structure. 
+ *
+ * Returns: the number of curves, or an error code.
+ * Since: 3.2.3
+ **/
+int
+gnutls_priority_mac_list (gnutls_priority_t pcache, const unsigned int** list)
+{
+  if (pcache->mac.algorithms == 0)
+    return 0;
+  
+  *list = pcache->mac.priority;
+  return pcache->mac.algorithms;
+}
+
 /**
  * gnutls_priority_compression_list:
  * @pcache: is a #gnutls_prioritity_t structure.
